/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/

package com.vektorsoft.demux.desktop.dlg;

import com.vektorsoft.demux.core.dlg.DMXDialog;
import com.vektorsoft.demux.core.dlg.DMXDialogView;
import com.vektorsoft.demux.core.dlg.DialogFactory;
import com.vektorsoft.demux.core.mva.DMXView;
import com.vektorsoft.demux.core.mva.DMXViewManager;
import com.vektorsoft.demux.core.resources.ResourceManager;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import javafx.stage.Stage;

/**
 * <p>
 * Dialog factory for JavaFX applications. It provides custom dialogs which can be used in DEMUX JavaFX 
 * applications. Since JavaFX currently does not have built-in dialogs, this class provides custom 
 * dialog implementation.
 * </p>
 * <p>
 *  Note that these dialogs are synchronous, meaning that they will block calling thread until they 
 * are closed.
 * </p>
 *
 * @author Vladimir Djurovic
 */
public class JFXDialogFactory implements DialogFactory {

    /**
     * View-to-dialog mappings.
     */
    private final Map<String, DMXDialog> dialogs;
    
    /** View manager instance. */
    private DMXViewManager viewManager;
    
    /**  Main stage for dialogs. */
    private Stage parent;
    
    /** Resource manager instance. */
    private ResourceManager resourceManager;

    /**
     * Create new instance.
     * 
     * @param manager view manager
     * @param parent  parent stage
     */
    public JFXDialogFactory(DMXViewManager manager, Stage parent) {
        this.viewManager = manager;
        this.parent = parent;
        dialogs = Collections.synchronizedMap(new HashMap<String, DMXDialog>());
    }

    /**
     * Sets resource manager.
     * 
     * @param resourceManager resource manager
     */
    public void setResourceManager(ResourceManager resourceManager) {
        this.resourceManager = resourceManager;
    }

    /**
     * <p>
     * Create dialog for JavaFX platform based on provided parameters. This
     * method will show view with specified ID inside a dialog window. Dialog
     * parameters (title, modality) are drawn from view definitions. Note that
     * this view should be an instance on {@link DMXDialogView}.
     * </p>
     * <p>
     * Clients can specify alternate dialog title by parameter {@code title},
     * which can be different from title defined within the view. If this
     * parameter is {@code null}, original title is used.
     * </p>
     *
     * @param viewId ID of view to display in dialog
     * @param title alternate title (different from one defined
     * @return dialog instance, or {@code null} if view with specified ID does
     * not exist
     */
    public JFXDialogStage createDialog(String viewId, String title) {
        JFXDialogStage dlg = null;
        DMXView view = viewManager.getView(viewId);
        if (view != null) {
            if (view instanceof DMXDialogView) {
                synchronized (dialogs) {
                    if (dialogs.containsKey(view.getViewId())) {
                        dlg = (JFXDialogStage) dialogs.get(view.getViewId());
                    } else {
                        dlg = new JFXDialogStage(parent, (DMXDialogView) view);
                        dialogs.put(viewId, dlg);
                    }
                }

                if (title != null) {
                    dlg.setTitle(title);
                }
            } else {
                throw new IllegalArgumentException("View is not instance of com.vektorsoft.demux.core.mva.DMXDialogView. View ID:" + viewId);
            }
        }
        return dlg;
    }

    /**
     * Create options dialog based on specified parameters. Optional dialogs
     * provide common functionality, similar to Swing {@code JOptionPane}.
     *
     * @param viewId ID of view to display, or {@code null} if view should not
     * be displayed
     * @param altTitle optional dialog title
     * @param options predefined options constants, defined in {@link DMXDialog}
     * @param content optional content to show in dialog
     * @return {@link DMXDialog} instance to display
     */
    public JFXOptionStage createOptionDialog(String viewId, String altTitle, int options, Object content) {
        JFXOptionStage dlg = null;
        if (viewId != null) {
            DMXView view = viewManager.getView(viewId);
            if (view instanceof DMXDialogView) {
                synchronized (dialogs) {
                    if (dialogs.containsKey(view.getViewId())) {
                        dlg = (JFXOptionStage) dialogs.get(view.getViewId());
                    } else {
                        dlg = new JFXOptionStage(parent, options, altTitle, view, resourceManager);
                        dialogs.put(viewId, dlg);
                    }
                }

            }
        } else {
            dlg = new JFXOptionStage(parent, options, altTitle, content, resourceManager);
        }

        return dlg;
    }
    
    /**
     * <p>
     * Creates options dialog with custom buttons. Buttons text is specified as array in {@code options}
     * parameter. Buttons are laid out in order specified, from left to right. The return value is index of 
     * selected button in {@code options} array. For example, in the following code:
     * 
     * <pre>
     *  <code>
     *      String[] options = {"Option 1", "Option 2", "Option 3"};
     *  </code>
     * </pre>
     * 
     *  If user selected {@code Option 2}, return value is 1 (option index in array).
     * </p>
     * <p>
     *  Parameter {@code viewId} is ID of the view which should be displayed inside dialog. If this parameter is null,
     * then parameter {@code content} is used as view content.
     * </p>
     * 
     * @param viewId ID of view to display
     * @param altTitle alternate title
     * @param options options for dialog
     * @param content content to display
     * @return dialog
     */
    public JFXOptionStage createOptionDialog(String viewId, String altTitle, String[] options, Object content){
         JFXOptionStage dlg = null;
        if (viewId != null) {
            DMXView view = viewManager.getView(viewId);
            if (view instanceof DMXDialogView) {
                synchronized (dialogs) {
                    if (dialogs.containsKey(view.getViewId())) {
                        dlg = (JFXOptionStage) dialogs.get(view.getViewId());
                    } else {
                        dlg = new JFXOptionStage(parent, options, altTitle, view, resourceManager);
                        dialogs.put(viewId, dlg);
                    }
                }

            }
        } else {
            dlg = new JFXOptionStage(parent, options, altTitle, content, resourceManager);
        }

        return dlg;
    }
    
    /**
     * <p>
     *  Creates dialog that will be used to display task progress. Progress display is handled by a view with 
     * specified ID (parameter {@code viewId}).
     * </p>
     * <p>
     *  This kind of dialog is shown automatically on task start, and removed when task is ended.
     * </p>
     * 
     * @param viewId view ID
     * @return progress dialog
     */
    public JFXProgressDialog createProgressDialog(String viewId) {
        JFXProgressDialog dlg = null;
        DMXView view = viewManager.getView(viewId);
        if (view != null && view instanceof DMXDialogView) {
            synchronized (dialogs) {
                if (dialogs.containsKey(view.getViewId())) {
                    dlg = (JFXProgressDialog) dialogs.get(view.getViewId());
                } else {
                    dlg = new JFXProgressDialog(parent, (DMXDialogView) view);
                    dialogs.put(viewId, dlg);
                }
            }


        } else {
            throw new IllegalArgumentException("View is not instance of com.vektorsoft.demux.core.mva.DMXDialogView. View ID:" + viewId);
        }
        return dlg;
    }
}
